@@ -1,45 +1,57 @@
-/* Copyright (C) 1999,2000,2001, 2003, 2005, 2006, 2009, 2010 Free Software Foundation, Inc.
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA
- */
+/* Copyright 1999-2001,2003,2005-2006,2009-2010,2012-2014,2017-2019
+     Free Software Foundation, Inc.
+
+   This file is part of Guile.
+
+   Guile is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published
+   by the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Guile is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+   License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with Guile.  If not, see
+   <https://www.gnu.org/licenses/>.  */
 
 
-
-/* Author: Mikael Djurfeldt <djurfeldt@nada.kth.se> */
+/* Original Author: Mikael Djurfeldt <djurfeldt@nada.kth.se> */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
-#include "libguile/_scm.h"
+#include <math.h>
+#include <stdio.h>
+#include <string.h>
+#include <sys/types.h>
+#include <unistd.h>
 
 #include <gmp.h>
-#include <stdio.h>
-#include <math.h>
-#include <string.h>
-#include "libguile/smob.h"
-#include "libguile/numbers.h"
-#include "libguile/feature.h"
-#include "libguile/strings.h"
-#include "libguile/arrays.h"
-#include "libguile/srfi-4.h"
-#include "libguile/vectors.h"
-#include "libguile/generalized-vectors.h"
 
-#include "libguile/validate.h"
-#include "libguile/random.h"
+#include "arrays.h"
+#include "feature.h"
+#include "generalized-arrays.h"
+#include "generalized-vectors.h"
+#include "gsubr.h"
+#include "list.h"
+#include "modules.h"
+#include "numbers.h"
+#include "numbers.h"
+#include "pairs.h"
+#include "smob.h"
+#include "srfi-4.h"
+#include "stime.h"
+#include "strings.h"
+#include "symbols.h"
+#include "variable.h"
+#include "vectors.h"
+
+#include "random.h"
+
 
 
 /*
@@ -73,8 +85,8 @@ scm_t_rng scm_the_rng;
 
 typedef struct scm_t_i_rstate {
   scm_t_rstate rstate;
-  scm_t_uint32 w;
-  scm_t_uint32 c;
+  uint32_t w;
+  uint32_t c;
 } scm_t_i_rstate;
 
 
@@ -84,12 +96,12 @@ typedef struct scm_t_i_rstate {
 #define M_PI 3.14159265359
 #endif
 
-static scm_t_uint32
+static uint32_t
 scm_i_uniform32 (scm_t_rstate *state)
 {
   scm_t_i_rstate *istate = (scm_t_i_rstate*) state;
-  scm_t_uint64 x = (scm_t_uint64) A * istate->w + istate->c;
-  scm_t_uint32 w = x & 0xffffffffUL;
+  uint64_t x = (uint64_t) A * istate->w + istate->c;
+  uint32_t w = x & 0xffffffffUL;
   istate->w = w;
   istate->c = x >> 32L;
   return w;
@@ -99,8 +111,8 @@ static void
 scm_i_init_rstate (scm_t_rstate *state, const char *seed, int n)
 {
   scm_t_i_rstate *istate = (scm_t_i_rstate*) state;
-  scm_t_uint32 w = 0L;
-  scm_t_uint32 c = 0L;
+  uint32_t w = 0L;
+  uint32_t c = 0L;
   int i, m;
   for (i = 0; i < n; ++i)
     {
@@ -133,7 +145,7 @@ scm_i_rstate_from_datum (scm_t_rstate *state, SCM value)
 #define FUNC_NAME "scm_i_rstate_from_datum"
 {
   scm_t_i_rstate *istate = (scm_t_i_rstate*) state;
-  scm_t_uint32 w, c;
+  uint32_t w, c;
   long length;
   
   SCM_VALIDATE_LIST_COPYLEN (SCM_ARG1, value, length);
@@ -241,8 +253,8 @@ scm_c_exp1 (scm_t_rstate *state)
 
 unsigned char scm_masktab[256];
 
-static inline scm_t_uint32
-scm_i_mask32 (scm_t_uint32 m)
+static inline uint32_t
+scm_i_mask32 (uint32_t m)
 {
   return (m < 0x100
 	  ? scm_masktab[m]
@@ -250,28 +262,28 @@ scm_i_mask32 (scm_t_uint32 m)
 	     ? scm_masktab[m >> 8] << 8 | 0xff
 	     : (m < 0x1000000
 		? scm_masktab[m >> 16] << 16 | 0xffff
-		: scm_masktab[m >> 24] << 24 | 0xffffff)));
+		: ((uint32_t) scm_masktab[m >> 24]) << 24 | 0xffffff)));
 }
 
-scm_t_uint32
-scm_c_random (scm_t_rstate *state, scm_t_uint32 m)
+uint32_t
+scm_c_random (scm_t_rstate *state, uint32_t m)
 {
-  scm_t_uint32 r, mask = scm_i_mask32 (m);
+  uint32_t r, mask = scm_i_mask32 (m);
   while ((r = state->rng->random_bits (state) & mask) >= m);
   return r;
 }
 
-scm_t_uint64
-scm_c_random64 (scm_t_rstate *state, scm_t_uint64 m)
+uint64_t
+scm_c_random64 (scm_t_rstate *state, uint64_t m)
 {
-  scm_t_uint64 r;
-  scm_t_uint32 mask;
+  uint64_t r;
+  uint32_t mask;
 
-  if (m <= SCM_T_UINT32_MAX)
-    return scm_c_random (state, (scm_t_uint32) m);
+  if (m <= UINT32_MAX)
+    return scm_c_random (state, (uint32_t) m);
   
   mask = scm_i_mask32 (m >> 32);
-  while ((r = ((scm_t_uint64) (state->rng->random_bits (state) & mask) << 32)
+  while ((r = ((uint64_t) (state->rng->random_bits (state) & mask) << 32)
           | state->rng->random_bits (state)) >= m)
     ;
   return r;
@@ -297,24 +309,24 @@ scm_c_random_bignum (scm_t_rstate *state, SCM m)
 {
   SCM result = scm_i_mkbig ();
   const size_t m_bits = mpz_sizeinbase (SCM_I_BIG_MPZ (m), 2);
-  /* how many bits would only partially fill the last scm_t_uint32? */
-  const size_t end_bits = m_bits % (sizeof (scm_t_uint32) * SCM_CHAR_BIT);
-  scm_t_uint32 *random_chunks = NULL;
-  const scm_t_uint32 num_full_chunks =
-    m_bits / (sizeof (scm_t_uint32) * SCM_CHAR_BIT);
-  const scm_t_uint32 num_chunks = num_full_chunks + ((end_bits) ? 1 : 0);
+  /* how many bits would only partially fill the last uint32_t? */
+  const size_t end_bits = m_bits % (sizeof (uint32_t) * SCM_CHAR_BIT);
+  uint32_t *random_chunks = NULL;
+  const uint32_t num_full_chunks =
+    m_bits / (sizeof (uint32_t) * SCM_CHAR_BIT);
+  const uint32_t num_chunks = num_full_chunks + ((end_bits) ? 1 : 0);
 
   /* we know the result will be this big */
   mpz_realloc2 (SCM_I_BIG_MPZ (result), m_bits);
 
   random_chunks =
-    (scm_t_uint32 *) scm_gc_calloc (num_chunks * sizeof (scm_t_uint32),
+    (uint32_t *) scm_gc_calloc (num_chunks * sizeof (uint32_t),
                                      "random bignum chunks");
 
   do
     {
-      scm_t_uint32 *current_chunk = random_chunks + (num_chunks - 1);
-      scm_t_uint32 chunks_left = num_chunks;
+      uint32_t *current_chunk = random_chunks + (num_chunks - 1);
+      uint32_t chunks_left = num_chunks;
 
       mpz_set_ui (SCM_I_BIG_MPZ (result), 0);
       
@@ -322,24 +334,24 @@ scm_c_random_bignum (scm_t_rstate *state, SCM m)
         {
           /* generate a mask with ones in the end_bits position, i.e. if
              end_bits is 3, then we'd have a mask of ...0000000111 */
-          const scm_t_uint32 rndbits = state->rng->random_bits (state);
-          int rshift = (sizeof (scm_t_uint32) * SCM_CHAR_BIT) - end_bits;
-          scm_t_uint32 mask = ((scm_t_uint32)-1) >> rshift;
-          scm_t_uint32 highest_bits = rndbits & mask;
+          const uint32_t rndbits = state->rng->random_bits (state);
+          int rshift = (sizeof (uint32_t) * SCM_CHAR_BIT) - end_bits;
+          uint32_t mask = ((uint32_t)-1) >> rshift;
+          uint32_t highest_bits = rndbits & mask;
           *current_chunk-- = highest_bits;
           chunks_left--;
         }
       
       while (chunks_left)
         {
-          /* now fill in the remaining scm_t_uint32 sized chunks */
+          /* now fill in the remaining uint32_t sized chunks */
           *current_chunk-- = state->rng->random_bits (state);
           chunks_left--;
         }
       mpz_import (SCM_I_BIG_MPZ (result),
                   num_chunks,
                   -1,
-                  sizeof (scm_t_uint32),
+                  sizeof (uint32_t),
                   0,
                   0,
                   random_chunks);
@@ -347,7 +359,7 @@ scm_c_random_bignum (scm_t_rstate *state, SCM m)
 	 all bits in order not to get a distorted distribution) */
     } while (mpz_cmp (SCM_I_BIG_MPZ (result), SCM_I_BIG_MPZ (m)) >= 0);
   scm_gc_free (random_chunks,
-               num_chunks * sizeof (scm_t_uint32),
+               num_chunks * sizeof (uint32_t),
                "random bignum chunks");
   return scm_i_normbig (result);
 }
@@ -369,7 +381,10 @@ make_rstate (scm_t_rstate *state)
  * Scheme level interface.
  */
 
-SCM_GLOBAL_VARIABLE_INIT (scm_var_random_state, "*random-state*", scm_seed_to_random_state (scm_from_locale_string ("URL:http://stat.fsu.edu/~geo/diehard.html")));
+SCM_GLOBAL_VARIABLE_INIT (scm_var_random_state, "*random-state*",
+                          scm_seed_to_random_state
+                          (scm_from_utf8_string
+                           ("URL:http://stat.fsu.edu/~geo/diehard.html")));
 
 SCM_DEFINE (scm_random, "random", 1, 1, 0, 
             (SCM n, SCM state),
@@ -396,15 +411,14 @@ SCM_DEFINE (scm_random, "random", 1, 1, 0,
       SCM_ASSERT_RANGE (1, n, SCM_I_INUM (n) > 0);
 #if SCM_SIZEOF_UINTPTR_T <= 4
       return scm_from_uint32 (scm_c_random (SCM_RSTATE (state),
-                                            (scm_t_uint32) m));
+                                            (uint32_t) m));
 #elif SCM_SIZEOF_UINTPTR_T <= 8
       return scm_from_uint64 (scm_c_random64 (SCM_RSTATE (state),
-                                              (scm_t_uint64) m));
+                                              (uint64_t) m));
 #else
 #error "Cannot deal with this platform's scm_t_bits size"
 #endif
     }
-  SCM_VALIDATE_NIM (1, n);
   if (SCM_REALP (n))
     return scm_from_double (SCM_REAL_VALUE (n)
 			    * scm_c_uniform01 (SCM_RSTATE (state)));
@@ -433,11 +447,33 @@ SCM_DEFINE (scm_seed_to_random_state, "seed->random-state", 1, 0, 0,
 #define FUNC_NAME s_scm_seed_to_random_state
 {
   SCM res;
+  char *c_str;
+  size_t len;
+
   if (SCM_NUMBERP (seed))
     seed = scm_number_to_string (seed, SCM_UNDEFINED);
   SCM_VALIDATE_STRING (1, seed);
-  res = make_rstate (scm_c_make_rstate (scm_i_string_chars (seed),
-					scm_i_string_length (seed)));
+
+  if (scm_i_is_narrow_string (seed))
+    /* This special case of a narrow string, where latin1 is used, is
+       for backward compatibility during the 2.2 stable series.  In
+       future major releases, we should use UTF-8 uniformly. */
+    c_str = scm_to_latin1_stringn (seed, &len);
+  else
+    c_str = scm_to_utf8_stringn (seed, &len);
+
+  /* 'scm_to_*_stringn' returns a 'size_t' for the length in bytes, but
+     'scm_c_make_rstate' accepts an 'int'.  Make sure the length fits in
+     an 'int'. */
+  if (len > INT_MAX)
+    {
+      free (c_str);
+      SCM_OUT_OF_RANGE (1, seed);
+    }
+
+  res = make_rstate (scm_c_make_rstate (c_str, len));
+  free (c_str);
+
   scm_remember_upto_here_1 (seed);
   return res;
   
@@ -496,63 +532,73 @@ SCM_DEFINE (scm_random_normal, "random:normal", 0, 1, 0,
 static void
 vector_scale_x (SCM v, double c)
 {
-  size_t n;
-  if (scm_is_simple_vector (v))
-    {
-      n = SCM_SIMPLE_VECTOR_LENGTH (v);
-      while (n-- > 0)
-	SCM_REAL_VALUE (SCM_SIMPLE_VECTOR_REF (v, n)) *= c;
-    }
-  else
-    {
-      /* must be a f64vector. */
-      scm_t_array_handle handle;
-      size_t i, len;
-      ssize_t inc;
-      double *elts;
+  scm_t_array_handle handle;
+  scm_t_array_dim const * dims;
+  ssize_t i, inc, ubnd;
 
-      elts = scm_f64vector_writable_elements (v, &handle, &len, &inc);
+  scm_array_get_handle (v, &handle);
+  dims = scm_array_handle_dims (&handle);
+  if (1 == scm_array_handle_rank (&handle))
+    {
+      ubnd = dims[0].ubnd;
+      inc = dims[0].inc;
 
-      for (i = 0; i < len; i++, elts += inc)
-	*elts *= c;
-      
-      scm_array_handle_release (&handle);
+      if (handle.element_type == SCM_ARRAY_ELEMENT_TYPE_F64)
+        {
+          double *elts = (double *)(handle.writable_elements) + handle.base;
+          for (i = dims[0].lbnd; i <= ubnd; ++i, elts += inc)
+            *elts *= c;
+          return;
+        }
+      else if (handle.element_type == SCM_ARRAY_ELEMENT_TYPE_SCM)
+        {
+          SCM *elts = (SCM *)(handle.writable_elements) + handle.base;
+          for (i = dims[0].lbnd; i <= ubnd; ++i, elts += inc)
+            SCM_REAL_VALUE (*elts) *= c;
+          return;
+        }
     }
+  scm_array_handle_release (&handle);
+  scm_misc_error (NULL, "must be a rank-1 array of type #t or 'f64", scm_list_1 (v));
 }
 
 static double
 vector_sum_squares (SCM v)
 {
   double x, sum = 0.0;
-  size_t n;
-  if (scm_is_simple_vector (v))
+  scm_t_array_handle handle;
+  scm_t_array_dim const * dims;
+  ssize_t i, inc, ubnd;
+
+  scm_array_get_handle (v, &handle);
+  dims = scm_array_handle_dims (&handle);
+  if (1 == scm_array_handle_rank (&handle))
     {
-      n = SCM_SIMPLE_VECTOR_LENGTH (v);
-      while (n-- > 0)
-	{
-	  x = SCM_REAL_VALUE (SCM_SIMPLE_VECTOR_REF (v, n));
-	  sum += x * x;
-	}
+      ubnd = dims[0].ubnd;
+      inc = dims[0].inc;
+      if (handle.element_type == SCM_ARRAY_ELEMENT_TYPE_F64)
+        {
+          const double *elts = (const double *)(handle.elements) + handle.base;
+          for (i = dims[0].lbnd; i <= ubnd; ++i, elts += inc)
+            {
+              x = *elts;
+              sum += x * x;
+            }
+          return sum;
+        }
+      else if (handle.element_type == SCM_ARRAY_ELEMENT_TYPE_SCM)
+        {
+          const SCM *elts = (const SCM *)(handle.elements) + handle.base;
+          for (i = dims[0].lbnd; i <= ubnd; ++i, elts += inc)
+            {
+              x = SCM_REAL_VALUE (*elts);
+              sum += x * x;
+            }
+          return sum;
+        }
     }
-  else
-    {
-      /* must be a f64vector. */
-      scm_t_array_handle handle;
-      size_t i, len;
-      ssize_t inc;
-      const double *elts;
-
-      elts = scm_f64vector_elements (v, &handle, &len, &inc);
-
-      for (i = 0; i < len; i++, elts += inc)
-	{
-	  x = *elts;
-	  sum += x * x;
-	}
-
-      scm_array_handle_release (&handle);
-    }
-  return sum;
+  scm_array_handle_release (&handle);
+  scm_misc_error (NULL, "must be an array of type #t or 'f64", scm_list_1 (v));
 }
 
 /* For the uniform distribution on the solid sphere, note that in
@@ -575,13 +621,13 @@ SCM_DEFINE (scm_random_solid_sphere_x, "random:solid-sphere!", 1, 1, 0,
   scm_random_normal_vector_x (v, state);
   vector_scale_x (v,
 		  pow (scm_c_uniform01 (SCM_RSTATE (state)),
-		       1.0 / scm_c_generalized_vector_length (v))
+		       1.0 / scm_c_array_length (v))
 		  / sqrt (vector_sum_squares (v)));
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
-SCM_DEFINE (scm_random_hollow_sphere_x, "random:hollow-sphere!", 1, 1, 0, 
+SCM_DEFINE (scm_random_hollow_sphere_x, "random:hollow-sphere!", 1, 1, 0,
             (SCM v, SCM state),
             "Fills vect with inexact real random numbers\n"
             "the sum of whose squares is equal to 1.0.\n"
@@ -601,45 +647,50 @@ SCM_DEFINE (scm_random_hollow_sphere_x, "random:hollow-sphere!", 1, 1, 0,
 #undef FUNC_NAME
 
 
-SCM_DEFINE (scm_random_normal_vector_x, "random:normal-vector!", 1, 1, 0, 
+SCM_DEFINE (scm_random_normal_vector_x, "random:normal-vector!", 1, 1, 0,
             (SCM v, SCM state),
             "Fills vect with inexact real random numbers that are\n"
             "independent and standard normally distributed\n"
             "(i.e., with mean 0 and variance 1).")
 #define FUNC_NAME s_scm_random_normal_vector_x
 {
-  long i;
   scm_t_array_handle handle;
-  scm_t_array_dim *dim;
+  scm_t_array_dim const * dims;
+  ssize_t i;
 
   if (SCM_UNBNDP (state))
     state = SCM_VARIABLE_REF (scm_var_random_state);
   SCM_VALIDATE_RSTATE (2, state);
 
-  scm_generalized_vector_get_handle (v, &handle);
-  dim = scm_array_handle_dims (&handle);
+  scm_array_get_handle (v, &handle);
+  if (1 != scm_array_handle_rank (&handle))
+    {
+      scm_array_handle_release (&handle);
+      scm_wrong_type_arg_msg (NULL, 0, v, "rank 1 array");
+    }
 
-  if (scm_is_vector (v))
+  dims = scm_array_handle_dims (&handle);
+
+  if (handle.element_type == SCM_ARRAY_ELEMENT_TYPE_SCM)
     {
       SCM *elts = scm_array_handle_writable_elements (&handle);
-      for (i = dim->lbnd; i <= dim->ubnd; i++, elts += dim->inc)
-	*elts = scm_from_double (scm_c_normal01 (SCM_RSTATE (state)));
+      for (i = dims->lbnd; i <= dims->ubnd; i++, elts += dims->inc)
+        *elts = scm_from_double (scm_c_normal01 (SCM_RSTATE (state)));
     }
   else
     {
       /* must be a f64vector. */
       double *elts = scm_array_handle_f64_writable_elements (&handle);
-      for (i = dim->lbnd; i <= dim->ubnd; i++, elts += dim->inc)
-	*elts = scm_c_normal01 (SCM_RSTATE (state));
+      for (i = dims->lbnd; i <= dims->ubnd; i++, elts += dims->inc)
+        *elts = scm_c_normal01 (SCM_RSTATE (state));
     }
 
   scm_array_handle_release (&handle);
-
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
-SCM_DEFINE (scm_random_exp, "random:exp", 0, 1, 0, 
+SCM_DEFINE (scm_random_exp, "random:exp", 0, 1, 0,
             (SCM state),
 	    "Return an inexact real in an exponential distribution with mean\n"
 	    "1.  For an exponential distribution with mean u use (* u\n"
@@ -665,7 +716,8 @@ random_state_of_last_resort (void)
   SCM time_of_day = scm_gettimeofday ();
   SCM sources = scm_list_n
     (scm_from_unsigned_integer (SCM_UNPACK (time_of_day)),  /* heap addr */
-     scm_getpid (),         /* process ID */
+     /* Avoid scm_getpid, since it depends on HAVE_POSIX. */
+     scm_from_unsigned_integer (getpid ()),                 /* process ID */
      scm_get_internal_real_time (), /* high-resolution process timer */
      scm_from_unsigned_integer ((scm_t_bits) &time_of_day), /* stack addr */
      scm_car (time_of_day), /* seconds since midnight 1970-01-01 UTC */
@@ -769,20 +821,14 @@ scm_init_random ()
     scm_i_rstate_to_datum
   };
   scm_the_rng = rng;
-  
+
   scm_tc16_rstate = scm_make_smob_type ("random-state", 0);
 
   for (m = 1; m <= 0x100; m <<= 1)
     for (i = m >> 1; i < m; ++i)
       scm_masktab[i] = m - 1;
-  
-#include "libguile/random.x"
+
+#include "random.x"
 
   scm_add_feature ("random");
 }
-
-/*
-  Local Variables:
-  c-file-style: "gnu"
-  End:
-*/
