@@ -29,12 +29,6 @@
   #:use-module (language cps)
   #:use-module (language cps slot-allocation)
   #:use-module (language cps utils)
-  #:use-module (language cps closure-conversion)
-  #:use-module (language cps loop-instrumentation)
-  #:use-module (language cps optimize)
-  #:use-module (language cps reify-primitives)
-  #:use-module (language cps renumber)
-  #:use-module (language cps split-rec)
   #:use-module (language cps intmap)
   #:use-module (language cps intset)
   #:use-module (system vm assembler)
@@ -162,6 +156,12 @@
          (emit-allocate-words asm (from-sp dst) (from-sp (slot nfields))))
         (($ $primcall 'allocate-words/immediate (annotation . nfields))
          (emit-allocate-words/immediate asm (from-sp dst) nfields))
+        (($ $primcall 'allocate-pointerless-words annotation (nfields))
+         (emit-allocate-pointerless-words asm (from-sp dst)
+                                       (from-sp (slot nfields))))
+        (($ $primcall 'allocate-pointerless-words/immediate
+            (annotation . nfields))
+         (emit-allocate-pointerless-words/immediate asm (from-sp dst) nfields))
         (($ $primcall 'scm-ref annotation (obj idx))
          (emit-scm-ref asm (from-sp dst) (from-sp (slot obj))
                        (from-sp (slot idx))))
@@ -186,8 +186,15 @@
          (emit-cache-ref asm (from-sp dst) key))
         (($ $primcall 'resolve-module public? (name))
          (emit-resolve-module asm (from-sp dst) (from-sp (slot name)) public?))
+        (($ $primcall 'module-variable #f (mod name))
+         (emit-module-variable asm (from-sp dst) (from-sp (slot mod))
+                               (from-sp (slot name))))
         (($ $primcall 'lookup #f (mod name))
-         (emit-lookup asm (from-sp dst) (from-sp (slot mod)) (from-sp (slot name))))
+         (emit-lookup asm (from-sp dst) (from-sp (slot mod))
+                      (from-sp (slot name))))
+        (($ $primcall 'lookup-bound #f (mod name))
+         (emit-lookup-bound asm (from-sp dst) (from-sp (slot mod))
+                            (from-sp (slot name))))
         (($ $primcall 'add/immediate y (x))
          (emit-add/immediate asm (from-sp dst) (from-sp (slot x)) y))
         (($ $primcall 'sub/immediate y (x))
@@ -380,6 +387,24 @@
         (#('throw/value+data param (val))
          (emit-throw/value+data asm (from-sp (slot val)) param))))
 
+    (define (emit-parallel-moves-after-return-and-reset-frame label nlocals)
+      (let lp ((moves (lookup-parallel-moves label allocation))
+               (reset-frame? #f))
+        (cond
+         ((and (not reset-frame?)
+               (and-map (match-lambda
+                         ((src . dst)
+                          (and (< src nlocals) (< dst nlocals))))
+                        moves))
+          (emit-reset-frame asm nlocals)
+          (lp moves #t))
+         (else
+          (match moves
+            (() #t)
+            (((src . dst) . moves)
+             (emit-fmov asm dst src)
+             (lp moves reset-frame?)))))))
+
     (define (compile-prompt label k kh escape? tag)
       (match (intmap-ref cps kh)
         (($ $kreceive ($ $arity req () rest () #f) khandler-body)
@@ -397,10 +422,7 @@
                         (($ $kargs names (_ ... rest))
                          (maybe-slot rest))))
              (emit-bind-rest asm (+ proc-slot nreq)))
-           (for-each (match-lambda
-                      ((src . dst) (emit-fmov asm dst src)))
-                     (lookup-parallel-moves kh allocation))
-           (emit-reset-frame asm frame-size)
+           (emit-parallel-moves-after-return-and-reset-frame kh frame-size)
            (emit-j asm (forward-label khandler-body))))))
 
     (define (compile-values label exp syms)
@@ -544,10 +566,7 @@
               (emit-receive-values asm proc-slot (->bool rest-var) nreq))
             (when (and rest-var (maybe-slot rest-var))
               (emit-bind-rest asm (+ proc-slot nreq)))
-            (for-each (match-lambda
-                       ((src . dst) (emit-fmov asm dst src)))
-                      (lookup-parallel-moves k allocation))
-            (emit-reset-frame asm frame-size)))))
+            (emit-parallel-moves-after-return-and-reset-frame k frame-size)))))
       (match exp
         (($ $call proc args)
          (do-call proc args
@@ -662,7 +681,7 @@
 
     (intmap-for-each compile-cont cps)))
 
-(define (emit-bytecode exp env opts)
+(define (compile-bytecode exp env opts)
   (let ((asm (make-assembler)))
     (intmap-for-each (lambda (kfun body)
                        (compile-function (intmap-select exp body) asm opts))
@@ -670,20 +689,3 @@
     (values (link-assembly asm #:page-aligned? (kw-arg-ref opts #:to-file? #f))
             env
             env)))
-
-(define (lower-cps exp opts)
-  ;; FIXME: For now the closure conversion pass relies on $rec instances
-  ;; being separated into SCCs.  We should fix this to not be the case,
-  ;; and instead move the split-rec pass back to
-  ;; optimize-higher-order-cps.
-  (set! exp (split-rec exp))
-  (set! exp (optimize-higher-order-cps exp opts))
-  (set! exp (convert-closures exp))
-  (set! exp (optimize-first-order-cps exp opts))
-  (set! exp (reify-primitives exp))
-  (set! exp (add-loop-instrumentation exp))
-  (renumber exp))
-
-(define (compile-bytecode exp env opts)
-  (set! exp (lower-cps exp opts))
-  (emit-bytecode exp env opts))
